// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "mozilla/glean/bindings/GleanJSMetricsLookup.h"

#include "mozilla/PerfectHash.h"
#include "mozilla/Maybe.h"
#include "mozilla/glean/bindings/MetricTypes.h"
#include "mozilla/glean/fog_ffi_generated.h"
#include "nsString.h"

#define GLEAN_INDEX_BITS (32)
#define GLEAN_TYPE_BITS (5)
#define GLEAN_ID_BITS (27)
#define GLEAN_TYPE_ID(id) ((id) >> GLEAN_ID_BITS)
#define GLEAN_METRIC_ID(id) ((id) & ((1ULL << GLEAN_ID_BITS) - 1))
#define GLEAN_OFFSET(entry) (entry & ((1ULL << GLEAN_INDEX_BITS) - 1))

namespace mozilla::glean {

// The category lookup table's entry type
using category_entry_t = uint32_t;
// The metric lookup table's entry type
// This is a bitpacked type with 32 bits available to index into
// the string table, 5 bits available to signify the metric type,
// and the remaining 27 bits devoted to 2 "signal"
// bits to signify important characteristics (metric's a labeled metric's
// submetric, metric's been registered at runtime) and 25 bits
// for built-in metric ids.
// Gives room for 33554432 of each combination of
// characteristics (which hopefully will prove to be enough).
using metric_entry_t = uint64_t;

static_assert(GLEAN_INDEX_BITS + GLEAN_TYPE_BITS + GLEAN_ID_BITS == sizeof(metric_entry_t) * 8, "Index, Type, and ID bits need to fit into a metric_entry_t");
static_assert(GLEAN_TYPE_BITS + GLEAN_ID_BITS <= sizeof(uint32_t) * 8, "Metric Types and IDs need to fit into at most 32 bits");
static_assert(66 < UINT32_MAX, "Too many metric categories generated.");
static_assert(404 < 33554432, "Too many metrics generated. Need room for 2 signal bits.");
static_assert(20 < 32, "Too many different metric types.");

already_AddRefed<GleanMetric> NewMetricFromId(uint32_t id, nsISupports* aParent) {
  uint32_t typeId = GLEAN_TYPE_ID(id);
  uint32_t metricId = GLEAN_METRIC_ID(id);

  switch (typeId) {
    case 1: /* string_list */
    {
      return MakeAndAddRef<GleanStringList>(metricId, aParent);
    }
    case 2: /* datetime */
    {
      return MakeAndAddRef<GleanDatetime>(metricId, aParent);
    }
    case 3: /* uuid */
    {
      return MakeAndAddRef<GleanUuid>(metricId, aParent);
    }
    case 4: /* boolean */
    {
      return MakeAndAddRef<GleanBoolean>(metricId, aParent);
    }
    case 5: /* string */
    {
      return MakeAndAddRef<GleanString>(metricId, aParent);
    }
    case 6: /* quantity */
    {
      return MakeAndAddRef<GleanQuantity>(metricId, aParent);
    }
    case 7: /* event */
    {
      return MakeAndAddRef<GleanEvent>(metricId, aParent);
    }
    case 8: /* text */
    {
      return MakeAndAddRef<GleanText>(metricId, aParent);
    }
    case 9: /* counter */
    {
      return MakeAndAddRef<GleanCounter>(metricId, aParent);
    }
    case 10: /* labeled_counter */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 10, aParent);
    }
    case 11: /* url */
    {
      return MakeAndAddRef<GleanUrl>(metricId, aParent);
    }
    case 12: /* timing_distribution */
    {
      return MakeAndAddRef<GleanTimingDistribution>(metricId, aParent);
    }
    case 13: /* denominator */
    {
      return MakeAndAddRef<GleanDenominator>(metricId, aParent);
    }
    case 14: /* numerator */
    {
      return MakeAndAddRef<GleanNumerator>(metricId, aParent);
    }
    case 15: /* custom_distribution */
    {
      return MakeAndAddRef<GleanCustomDistribution>(metricId, aParent);
    }
    case 16: /* labeled_boolean */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 16, aParent);
    }
    case 17: /* timespan */
    {
      return MakeAndAddRef<GleanTimespan>(metricId, aParent);
    }
    case 18: /* memory_distribution */
    {
      return MakeAndAddRef<GleanMemoryDistribution>(metricId, aParent);
    }
    case 19: /* labeled_string */
    {
      return MakeAndAddRef<GleanLabeled>(metricId, 19, aParent);
    }
    case 20: /* rate */
    {
      return MakeAndAddRef<GleanRate>(metricId, aParent);
    }
    default:
    MOZ_ASSERT_UNREACHABLE("Invalid type ID reached when trying to instantiate a new metric");
    return nullptr;
  }
}

/**
 * Create a submetric instance for a labeled metric of the provided type and id for the given label.
 * Assigns or retrieves an id for the submetric from the SDK.
 *
 * @param aParentTypeId - The type of the parent labeled metric identified as a number generated during codegen.
 *                        Only used to identify which X of LabeledX you are so that X can be created here.
 * @param aParentMetricId - The metric id for the parent labeled metric.
 * @param aLabel - The label for the submetric. Might not adhere to the SDK label format.
 * @param aSubmetricId - an outparam which is assigned the submetric's SDK-generated submetric id.
 *                       Used only by GIFFT.
 */
already_AddRefed<GleanMetric> NewSubMetricFromIds(uint32_t aParentTypeId,
                                           uint32_t aParentMetricId,
                                           const nsACString& aLabel,
                                           uint32_t* aSubmetricId,
                                           nsISupports* aParent) {
  switch (aParentTypeId) {
    case 10: { /* labeled_counter */
      auto id = impl::fog_labeled_counter_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanCounter>(id, aParent);
    }
    case 16: { /* labeled_boolean */
      auto id = impl::fog_labeled_boolean_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanBoolean>(id, aParent);
    }
    case 19: { /* labeled_string */
      auto id = impl::fog_labeled_string_get(aParentMetricId, &aLabel);
      *aSubmetricId = id;
      return MakeAndAddRef<GleanString>(id, aParent);
    }
    default: {
      MOZ_ASSERT_UNREACHABLE("Invalid type ID for submetric.");
      return nullptr;
    }
  }
}

static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry);
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, metric_entry_t entry);

#if defined(_MSC_VER) && !defined(__clang__)
const char gCategoryStringTable[] = {
#else
constexpr char gCategoryStringTable[] = {
#endif
  /*     0 - "backgroundUpdate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*    17 - "browserLaunchedToHandle" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '\0',
  /*    41 - "browserMigration" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*    58 - "messagingSystem" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '\0',
  /*    74 - "messagingSystemAttribution" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '\0',
  /*   101 - "newtab" */ 'n', 'e', 'w', 't', 'a', 'b', '\0',
  /*   108 - "newtabSearch" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '\0',
  /*   121 - "pocket" */ 'p', 'o', 'c', 'k', 'e', 't', '\0',
  /*   128 - "topSites" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '\0',
  /*   137 - "topsites" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '\0',
  /*   146 - "privacyUiFppClick" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '\0',
  /*   164 - "newtabSearchAd" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '\0',
  /*   179 - "serp" */ 's', 'e', 'r', 'p', '\0',
  /*   184 - "shopping" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '\0',
  /*   193 - "shoppingSettings" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '\0',
  /*   210 - "quickSuggest" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '\0',
  /*   223 - "urlbar" */ 'u', 'r', 'l', 'b', 'a', 'r', '\0',
  /*   230 - "browserEngagement" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*   248 - "pingCentre" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '\0',
  /*   259 - "gmp" */ 'g', 'm', 'p', '\0',
  /*   263 - "rtcrtpsender" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '\0',
  /*   276 - "rtcrtpsenderSetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*   302 - "bloburl" */ 'b', 'l', 'o', 'b', 'u', 'r', 'l', '\0',
  /*   310 - "perf" */ 'p', 'e', 'r', 'f', '\0',
  /*   315 - "paint" */ 'p', 'a', 'i', 'n', 't', '\0',
  /*   321 - "wr" */ 'w', 'r', '\0',
  /*   324 - "dotprint" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '\0',
  /*   333 - "networking" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '\0',
  /*   344 - "netwerk" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '\0',
  /*   352 - "network" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '\0',
  /*   360 - "dataStorage" */ 'd', 'a', 't', 'a', 'S', 't', 'o', 'r', 'a', 'g', 'e', '\0',
  /*   372 - "backgroundTasksRmdirBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '\0',
  /*   397 - "backgroundTasksRmdirHttpCache" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '\0',
  /*   427 - "backgroundTasksRmdirQuota" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '\0',
  /*   453 - "cookieBanners" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '\0',
  /*   467 - "cookieBannersClick" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '\0',
  /*   486 - "crash" */ 'c', 'r', 'a', 's', 'h', '\0',
  /*   492 - "extensions" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   503 - "extensionsApisDnr" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '\0',
  /*   521 - "extensionsQuarantinedDomains" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '\0',
  /*   550 - "formautofillCreditcards" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '\0',
  /*   574 - "fog" */ 'f', 'o', 'g', '\0',
  /*   578 - "fogIpc" */ 'f', 'o', 'g', 'I', 'p', 'c', '\0',
  /*   585 - "fogValidation" */ 'f', 'o', 'g', 'V', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '\0',
  /*   599 - "testOnly" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '\0',
  /*   608 - "testOnlyIpc" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '\0',
  /*   620 - "testOnlyJog" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '\0',
  /*   632 - "nimbusEvents" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '\0',
  /*   645 - "pwmgr" */ 'p', 'w', 'm', 'g', 'r', '\0',
  /*   651 - "pdfjs" */ 'p', 'd', 'f', 'j', 's', '\0',
  /*   657 - "power" */ 'p', 'o', 'w', 'e', 'r', '\0',
  /*   663 - "powerBattery" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '\0',
  /*   676 - "powerCpuMsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   696 - "powerWakeupsPerThread" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
  /*   718 - "fingerprintingProtection" */ 'f', 'i', 'n', 'g', 'e', 'r', 'p', 'r', 'i', 'n', 't', 'i', 'n', 'g', 'P', 'r', 'o', 't', 'e', 'c', 't', 'i', 'o', 'n', '\0',
  /*   743 - "searchEngineDefault" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '\0',
  /*   763 - "searchEnginePrivate" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '\0',
  /*   783 - "searchService" */ 's', 'e', 'a', 'r', 'c', 'h', 'S', 'e', 'r', 'v', 'i', 'c', 'e', '\0',
  /*   797 - "dap" */ 'd', 'a', 'p', '\0',
  /*   801 - "legacyTelemetry" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '\0',
  /*   817 - "translations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*   830 - "translationsPanel" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '\0',
  /*   848 - "firstStartup" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '\0',
  /*   861 - "blocklist" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '\0',
  /*   871 - "update" */ 'u', 'p', 'd', 'a', 't', 'e', '\0',
  /*   878 - "timerThread" */ 't', 'i', 'm', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '\0',
};


static_assert(sizeof(gCategoryStringTable) < UINT32_MAX, "Category string table is too large.");

const category_entry_t sCategoryByNameLookupEntries[] = {
  871ul,
  676ul,
  230ul,
  817ul,
  783ul,
  620ul,
  372ul,
  663ul,
  467ul,
  74ul,
  352ul,
  848ul,
  453ul,
  128ul,
  696ul,
  657ul,
  248ul,
  797ul,
  360ul,
  223ul,
  492ul,
  344ul,
  321ul,
  743ul,
  17ul,
  210ul,
  718ul,
  184ul,
  550ul,
  263ul,
  830ul,
  578ul,
  302ul,
  397ul,
  310ul,
  521ul,
  503ul,
  179ul,
  486ul,
  608ul,
  193ul,
  137ul,
  146ul,
  801ul,
  101ul,
  333ul,
  861ul,
  645ul,
  651ul,
  259ul,
  108ul,
  324ul,
  599ul,
  574ul,
  276ul,
  41ul,
  632ul,
  427ul,
  763ul,
  585ul,
  58ul,
  0ul,
  164ul,
  315ul,
  878ul,
  121ul
};



Maybe<uint32_t>
CategoryByNameLookup(const nsACString& aKey)
{
  static const uint8_t BASES[] = {
       0,   0,   0,   0,   0,   0,   0,   1,   1,   0,   0,   0,   1,   0,   0,   0,
       1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   1,
       0,   0,   0,   0,   1,   0,   1,   1,   0,   0,   0,   0,   0,   0,   0,   0,
       1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,
       0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   2,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   2,   0,   0,   0,   0,   0,   2,   0,   0,   0,   1,
       0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,
       2,   3,   0,   6,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   5,   0,   0,   1,   0,   0,   0,   0,   1,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   4,   1,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       6,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   2,   0,   0,   0,   1,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   5,   0,
       0,   0,   0,   8,   0,   0,   0,   3,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   0,   1,   0,   0,   0,   0,   3,   0,   0,   0,   0,   0,   1,   2,   0,
       0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   1,   0,   0,   4,   0,   0,
       0,   0,   0,   0,   0,   0,  10,   0,   0,   0,   0,   0,   6,   0,   0,   0,
       0,   0,  29,   0,   0,   0,   0,   0,   0,   0,   0,   0,  36,   0,   0,   0,
       0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  11,   0,
       0,   0,   2,   0,   0,   1,   0,   0,   3,   0,   0,   0,   0,   0,   0,  17,
       0,   0,   0,   0,   0,   0,  54,   0,   0,   0,   0,   0,   0,   0,   0,   0,
       0,   6,   0,   0,   0,   0,   0,  54,   0,   0,   0,   0,   0,  21,   0,   0,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sCategoryByNameLookupEntries);
  return category_result_check(aKey, entry);
}


#if defined(_MSC_VER) && !defined(__clang__)
const char gMetricStringTable[] = {
#else
constexpr char gMetricStringTable[] = {
#endif
  /*     0 - "backgroundUpdate.reasonsToNotUpdate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'a', 's', 'o', 'n', 's', 'T', 'o', 'N', 'o', 't', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /*    36 - "backgroundUpdate.timeLastUpdateScheduled" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'i', 'm', 'e', 'L', 'a', 's', 't', 'U', 'p', 'd', 'a', 't', 'e', 'S', 'c', 'h', 'e', 'd', 'u', 'l', 'e', 'd', '\0',
  /*    77 - "backgroundUpdate.clientId" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*   103 - "backgroundUpdate.exitCodeException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*   138 - "backgroundUpdate.exitCodeSuccess" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'e', 'x', 'i', 't', 'C', 'o', 'd', 'e', 'S', 'u', 'c', 'c', 'e', 's', 's', '\0',
  /*   171 - "backgroundUpdate.finalState" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'f', 'i', 'n', 'a', 'l', 'S', 't', 'a', 't', 'e', '\0',
  /*   199 - "backgroundUpdate.reasons" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'a', 's', 'o', 'n', 's', '\0',
  /*   224 - "backgroundUpdate.registeredRestartAttempted" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'g', 'i', 's', 't', 'e', 'r', 'e', 'd', 'R', 'e', 's', 't', 'a', 'r', 't', 'A', 't', 't', 'e', 'm', 'p', 't', 'e', 'd', '\0',
  /*   268 - "backgroundUpdate.registeredRestartSuccess" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 'r', 'e', 'g', 'i', 's', 't', 'e', 'r', 'e', 'd', 'R', 'e', 's', 't', 'a', 'r', 't', 'S', 'u', 'c', 'c', 'e', 's', 's', '\0',
  /*   310 - "backgroundUpdate.states" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 's', 't', 'a', 't', 'e', 's', '\0',
  /*   334 - "backgroundUpdate.targetingEnvCurrentDate" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'C', 'u', 'r', 'r', 'e', 'n', 't', 'D', 'a', 't', 'e', '\0',
  /*   375 - "backgroundUpdate.targetingEnvFirefoxVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'F', 'i', 'r', 'e', 'f', 'o', 'x', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   419 - "backgroundUpdate.targetingEnvProfileAge" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'n', 'v', 'P', 'r', 'o', 'f', 'i', 'l', 'e', 'A', 'g', 'e', '\0',
  /*   459 - "backgroundUpdate.targetingException" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'c', 'e', 'p', 't', 'i', 'o', 'n', '\0',
  /*   495 - "backgroundUpdate.targetingExists" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'E', 'x', 'i', 's', 't', 's', '\0',
  /*   528 - "backgroundUpdate.targetingVersion" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '.', 't', 'a', 'r', 'g', 'e', 't', 'i', 'n', 'g', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   562 - "browserLaunchedToHandle.systemNotification" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'L', 'a', 'u', 'n', 'c', 'h', 'e', 'd', 'T', 'o', 'H', 'a', 'n', 'd', 'l', 'e', '.', 's', 'y', 's', 't', 'e', 'm', 'N', 'o', 't', 'i', 'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', '\0',
  /*   605 - "browserMigration.matchedExtensions" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '.', 'm', 'a', 't', 'c', 'h', 'e', 'd', 'E', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   640 - "browserMigration.unmatchedExtensions" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'M', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '.', 'u', 'n', 'm', 'a', 't', 'c', 'h', 'e', 'd', 'E', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '\0',
  /*   677 - "messagingSystem.action" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'a', 'c', 't', 'i', 'o', 'n', '\0',
  /*   700 - "messagingSystem.addonVersion" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'a', 'd', 'd', 'o', 'n', 'V', 'e', 'r', 's', 'i', 'o', 'n', '\0',
  /*   729 - "messagingSystem.browserSessionId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'b', 'r', 'o', 'w', 's', 'e', 'r', 'S', 'e', 's', 's', 'i', 'o', 'n', 'I', 'd', '\0',
  /*   762 - "messagingSystem.bucketId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'b', 'u', 'c', 'k', 'e', 't', 'I', 'd', '\0',
  /*   787 - "messagingSystem.cfrAction" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'c', 'f', 'r', 'A', 'c', 't', 'i', 'o', 'n', '\0',
  /*   813 - "messagingSystem.clientId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /*   838 - "messagingSystem.event" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', '\0',
  /*   860 - "messagingSystem.eventContext" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', '\0',
  /*   889 - "messagingSystem.eventContextParseError" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'C', 'o', 'n', 't', 'e', 'x', 't', 'P', 'a', 'r', 's', 'e', 'E', 'r', 'r', 'o', 'r', '\0',
  /*   928 - "messagingSystem.eventPage" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'P', 'a', 'g', 'e', '\0',
  /*   954 - "messagingSystem.eventReason" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'R', 'e', 'a', 's', 'o', 'n', '\0',
  /*   982 - "messagingSystem.eventSource" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'e', 'v', 'e', 'n', 't', 'S', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1010 - "messagingSystem.gleanPingForPingFailures" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'g', 'l', 'e', 'a', 'n', 'P', 'i', 'n', 'g', 'F', 'o', 'r', 'P', 'i', 'n', 'g', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  1051 - "messagingSystem.impressionId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', 'I', 'd', '\0',
  /*  1080 - "messagingSystem.invalidNestedData" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'i', 'n', 'v', 'a', 'l', 'i', 'd', 'N', 'e', 's', 't', 'e', 'd', 'D', 'a', 't', 'a', '\0',
  /*  1114 - "messagingSystem.locale" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'l', 'o', 'c', 'a', 'l', 'e', '\0',
  /*  1137 - "messagingSystem.messageId" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'm', 'e', 's', 's', 'a', 'g', 'e', 'I', 'd', '\0',
  /*  1163 - "messagingSystem.page" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'p', 'a', 'g', 'e', '\0',
  /*  1184 - "messagingSystem.pingType" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  1209 - "messagingSystem.source" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1232 - "messagingSystem.unknownKeyCount" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  1264 - "messagingSystem.unknownKeys" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 's', '\0',
  /*  1292 - "messagingSystemAttribution.campaign" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'c', 'a', 'm', 'p', 'a', 'i', 'g', 'n', '\0',
  /*  1328 - "messagingSystemAttribution.content" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', '\0',
  /*  1363 - "messagingSystemAttribution.dlsource" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'd', 'l', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1399 - "messagingSystemAttribution.dltoken" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'd', 'l', 't', 'o', 'k', 'e', 'n', '\0',
  /*  1434 - "messagingSystemAttribution.experiment" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'e', 'x', 'p', 'e', 'r', 'i', 'm', 'e', 'n', 't', '\0',
  /*  1472 - "messagingSystemAttribution.medium" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'm', 'e', 'd', 'i', 'u', 'm', '\0',
  /*  1506 - "messagingSystemAttribution.msstoresignedin" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'm', 's', 's', 't', 'o', 'r', 'e', 's', 'i', 'g', 'n', 'e', 'd', 'i', 'n', '\0',
  /*  1549 - "messagingSystemAttribution.source" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  1583 - "messagingSystemAttribution.ua" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'u', 'a', '\0',
  /*  1613 - "messagingSystemAttribution.unknownKeys" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'u', 'n', 'k', 'n', 'o', 'w', 'n', 'K', 'e', 'y', 's', '\0',
  /*  1652 - "messagingSystemAttribution.variation" */ 'm', 'e', 's', 's', 'a', 'g', 'i', 'n', 'g', 'S', 'y', 's', 't', 'e', 'm', 'A', 't', 't', 'r', 'i', 'b', 'u', 't', 'i', 'o', 'n', '.', 'v', 'a', 'r', 'i', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1689 - "newtab.blockedSponsors" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'b', 'l', 'o', 'c', 'k', 'e', 'd', 'S', 'p', 'o', 'n', 's', 'o', 'r', 's', '\0',
  /*  1712 - "newtab.closed" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'c', 'l', 'o', 's', 'e', 'd', '\0',
  /*  1726 - "newtab.homepageCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'h', 'o', 'm', 'e', 'p', 'a', 'g', 'e', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  1750 - "newtab.locale" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'l', 'o', 'c', 'a', 'l', 'e', '\0',
  /*  1764 - "newtab.newtabCategory" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'n', 'e', 'w', 't', 'a', 'b', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  1786 - "newtab.opened" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 'o', 'p', 'e', 'n', 'e', 'd', '\0',
  /*  1800 - "newtab.sovAllocation" */ 'n', 'e', 'w', 't', 'a', 'b', '.', 's', 'o', 'v', 'A', 'l', 'l', 'o', 'c', 'a', 't', 'i', 'o', 'n', '\0',
  /*  1821 - "newtabSearch.enabled" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1842 - "newtabSearch.issued" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', '.', 'i', 's', 's', 'u', 'e', 'd', '\0',
  /*  1862 - "pocket.click" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  1875 - "pocket.enabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1890 - "pocket.impression" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  1908 - "pocket.isSignedIn" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 'i', 's', 'S', 'i', 'g', 'n', 'e', 'd', 'I', 'n', '\0',
  /*  1926 - "pocket.save" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'a', 'v', 'e', '\0',
  /*  1938 - "pocket.sponsoredStoriesEnabled" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'S', 't', 'o', 'r', 'i', 'e', 's', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  1969 - "pocket.topicClick" */ 'p', 'o', 'c', 'k', 'e', 't', '.', 't', 'o', 'p', 'i', 'c', 'C', 'l', 'i', 'c', 'k', '\0',
  /*  1987 - "topSites.advertiser" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'a', 'd', 'v', 'e', 'r', 't', 'i', 's', 'e', 'r', '\0',
  /*  2007 - "topSites.contextId" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'c', 'o', 'n', 't', 'e', 'x', 't', 'I', 'd', '\0',
  /*  2026 - "topSites.pingType" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  2044 - "topSites.position" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n', '\0',
  /*  2062 - "topSites.reportingUrl" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 'r', 'e', 'p', 'o', 'r', 't', 'i', 'n', 'g', 'U', 'r', 'l', '\0',
  /*  2084 - "topSites.source" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  2100 - "topSites.tileId" */ 't', 'o', 'p', 'S', 'i', 't', 'e', 's', '.', 't', 'i', 'l', 'e', 'I', 'd', '\0',
  /*  2116 - "topsites.click" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2131 - "topsites.enabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2148 - "topsites.impression" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2168 - "topsites.rows" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 'r', 'o', 'w', 's', '\0',
  /*  2182 - "topsites.sponsoredEnabled" */ 't', 'o', 'p', 's', 'i', 't', 'e', 's', '.', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /*  2208 - "privacyUiFppClick.checkbox" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '.', 'c', 'h', 'e', 'c', 'k', 'b', 'o', 'x', '\0',
  /*  2235 - "privacyUiFppClick.menu" */ 'p', 'r', 'i', 'v', 'a', 'c', 'y', 'U', 'i', 'F', 'p', 'p', 'C', 'l', 'i', 'c', 'k', '.', 'm', 'e', 'n', 'u', '\0',
  /*  2258 - "newtabSearchAd.click" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'c', 'l', 'i', 'c', 'k', '\0',
  /*  2279 - "newtabSearchAd.impression" */ 'n', 'e', 'w', 't', 'a', 'b', 'S', 'e', 'a', 'r', 'c', 'h', 'A', 'd', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2305 - "serp.abandonment" */ 's', 'e', 'r', 'p', '.', 'a', 'b', 'a', 'n', 'd', 'o', 'n', 'm', 'e', 'n', 't', '\0',
  /*  2322 - "serp.adImpression" */ 's', 'e', 'r', 'p', '.', 'a', 'd', 'I', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2340 - "serp.categorizationDuration" */ 's', 'e', 'r', 'p', '.', 'c', 'a', 't', 'e', 'g', 'o', 'r', 'i', 'z', 'a', 't', 'i', 'o', 'n', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  2368 - "serp.engagement" */ 's', 'e', 'r', 'p', '.', 'e', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*  2384 - "serp.impression" */ 's', 'e', 'r', 'p', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  2400 - "shopping.addressBarFeatureCalloutDisplayed" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 'a', 'd', 'd', 'r', 'e', 's', 's', 'B', 'a', 'r', 'F', 'e', 'a', 't', 'u', 'r', 'e', 'C', 'a', 'l', 'l', 'o', 'u', 't', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'e', 'd', '\0',
  /*  2443 - "shopping.addressBarIconClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 'a', 'd', 'd', 'r', 'e', 's', 's', 'B', 'a', 'r', 'I', 'c', 'o', 'n', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2474 - "shopping.addressBarIconDisplayed" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 'a', 'd', 'd', 'r', 'e', 's', 's', 'B', 'a', 'r', 'I', 'c', 'o', 'n', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'e', 'd', '\0',
  /*  2507 - "shopping.productPageVisits" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 'p', 'r', 'o', 'd', 'u', 'c', 't', 'P', 'a', 'g', 'e', 'V', 'i', 's', 'i', 't', 's', '\0',
  /*  2534 - "shopping.surfaceAnalyzeReviewsNoneAvailableClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'A', 'n', 'a', 'l', 'y', 'z', 'e', 'R', 'e', 'v', 'i', 'e', 'w', 's', 'N', 'o', 'n', 'e', 'A', 'v', 'a', 'i', 'l', 'a', 'b', 'l', 'e', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2585 - "shopping.surfaceClosed" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'C', 'l', 'o', 's', 'e', 'd', '\0',
  /*  2608 - "shopping.surfaceDisplayed" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'e', 'd', '\0',
  /*  2634 - "shopping.surfaceLearnMoreClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'L', 'e', 'a', 'r', 'n', 'M', 'o', 'r', 'e', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2667 - "shopping.surfaceNoReviewReliabilityAvailable" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'N', 'o', 'R', 'e', 'v', 'i', 'e', 'w', 'R', 'e', 'l', 'i', 'a', 'b', 'i', 'l', 'i', 't', 'y', 'A', 'v', 'a', 'i', 'l', 'a', 'b', 'l', 'e', '\0',
  /*  2712 - "shopping.surfaceNotNowClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'N', 'o', 't', 'N', 'o', 'w', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2742 - "shopping.surfaceOnboardingDisplayed" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'O', 'n', 'b', 'o', 'a', 'r', 'd', 'i', 'n', 'g', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'e', 'd', '\0',
  /*  2778 - "shopping.surfaceOptInClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'O', 'p', 't', 'I', 'n', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2807 - "shopping.surfacePoweredByFakespotLinkClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'P', 'o', 'w', 'e', 'r', 'e', 'd', 'B', 'y', 'F', 'a', 'k', 'e', 's', 'p', 'o', 't', 'L', 'i', 'n', 'k', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2852 - "shopping.surfaceReactivatedButtonClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'R', 'e', 'a', 'c', 't', 'i', 'v', 'a', 't', 'e', 'd', 'B', 'u', 't', 't', 'o', 'n', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2893 - "shopping.surfaceReanalyzeClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'R', 'e', 'a', 'n', 'a', 'l', 'y', 'z', 'e', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2926 - "shopping.surfaceSettingsExpandClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'E', 'x', 'p', 'a', 'n', 'd', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  2964 - "shopping.surfaceShowMoreReviewsButtonClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'M', 'o', 'r', 'e', 'R', 'e', 'v', 'i', 'e', 'w', 's', 'B', 'u', 't', 't', 'o', 'n', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3009 - "shopping.surfaceShowPrivacyPolicyClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'P', 'r', 'i', 'v', 'a', 'c', 'y', 'P', 'o', 'l', 'i', 'c', 'y', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3050 - "shopping.surfaceShowQualityExplainerClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'Q', 'u', 'a', 'l', 'i', 't', 'y', 'E', 'x', 'p', 'l', 'a', 'i', 'n', 'e', 'r', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3094 - "shopping.surfaceShowQualityExplainerUrlClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'Q', 'u', 'a', 'l', 'i', 't', 'y', 'E', 'x', 'p', 'l', 'a', 'i', 'n', 'e', 'r', 'U', 'r', 'l', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3141 - "shopping.surfaceShowTermsClicked" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 'h', 'o', 'w', 'T', 'e', 'r', 'm', 's', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3174 - "shopping.surfaceStaleAnalysisShown" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '.', 's', 'u', 'r', 'f', 'a', 'c', 'e', 'S', 't', 'a', 'l', 'e', 'A', 'n', 'a', 'l', 'y', 's', 'i', 's', 'S', 'h', 'o', 'w', 'n', '\0',
  /*  3209 - "shoppingSettings.componentOptedOut" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'c', 'o', 'm', 'p', 'o', 'n', 'e', 'n', 't', 'O', 'p', 't', 'e', 'd', 'O', 'u', 't', '\0',
  /*  3244 - "shoppingSettings.hasOnboarded" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'h', 'a', 's', 'O', 'n', 'b', 'o', 'a', 'r', 'd', 'e', 'd', '\0',
  /*  3274 - "shoppingSettings.nimbusDisabledShopping" */ 's', 'h', 'o', 'p', 'p', 'i', 'n', 'g', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', '.', 'n', 'i', 'm', 'b', 'u', 's', 'D', 'i', 's', 'a', 'b', 'l', 'e', 'd', 'S', 'h', 'o', 'p', 'p', 'i', 'n', 'g', '\0',
  /*  3314 - "quickSuggest.advertiser" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'a', 'd', 'v', 'e', 'r', 't', 'i', 's', 'e', 'r', '\0',
  /*  3338 - "quickSuggest.blockId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'b', 'l', 'o', 'c', 'k', 'I', 'd', '\0',
  /*  3359 - "quickSuggest.contextId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'c', 'o', 'n', 't', 'e', 'x', 't', 'I', 'd', '\0',
  /*  3382 - "quickSuggest.iabCategory" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 'a', 'b', 'C', 'a', 't', 'e', 'g', 'o', 'r', 'y', '\0',
  /*  3407 - "quickSuggest.improveSuggestExperience" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 'm', 'p', 'r', 'o', 'v', 'e', 'S', 'u', 'g', 'g', 'e', 's', 't', 'E', 'x', 'p', 'e', 'r', 'i', 'e', 'n', 'c', 'e', '\0',
  /*  3445 - "quickSuggest.isClicked" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'i', 's', 'C', 'l', 'i', 'c', 'k', 'e', 'd', '\0',
  /*  3468 - "quickSuggest.matchType" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'm', 'a', 't', 'c', 'h', 'T', 'y', 'p', 'e', '\0',
  /*  3491 - "quickSuggest.pingType" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'p', 'i', 'n', 'g', 'T', 'y', 'p', 'e', '\0',
  /*  3513 - "quickSuggest.position" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n', '\0',
  /*  3535 - "quickSuggest.reportingUrl" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'r', 'e', 'p', 'o', 'r', 't', 'i', 'n', 'g', 'U', 'r', 'l', '\0',
  /*  3561 - "quickSuggest.requestId" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 'I', 'd', '\0',
  /*  3584 - "quickSuggest.source" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 's', 'o', 'u', 'r', 'c', 'e', '\0',
  /*  3604 - "quickSuggest.suggestedIndex" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 's', 'u', 'g', 'g', 'e', 's', 't', 'e', 'd', 'I', 'n', 'd', 'e', 'x', '\0',
  /*  3632 - "quickSuggest.suggestedIndexRelativeToGroup" */ 'q', 'u', 'i', 'c', 'k', 'S', 'u', 'g', 'g', 'e', 's', 't', '.', 's', 'u', 'g', 'g', 'e', 's', 't', 'e', 'd', 'I', 'n', 'd', 'e', 'x', 'R', 'e', 'l', 'a', 't', 'i', 'v', 'e', 'T', 'o', 'G', 'r', 'o', 'u', 'p', '\0',
  /*  3675 - "urlbar.abandonment" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'a', 'b', 'a', 'n', 'd', 'o', 'n', 'm', 'e', 'n', 't', '\0',
  /*  3694 - "urlbar.engagement" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'e', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '\0',
  /*  3712 - "urlbar.exposure" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'e', 'x', 'p', 'o', 's', 'u', 'r', 'e', '\0',
  /*  3728 - "urlbar.impression" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'i', 'm', 'p', 'r', 'e', 's', 's', 'i', 'o', 'n', '\0',
  /*  3746 - "urlbar.prefMaxResults" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'M', 'a', 'x', 'R', 'e', 's', 'u', 'l', 't', 's', '\0',
  /*  3768 - "urlbar.prefSuggestDataCollection" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'D', 'a', 't', 'a', 'C', 'o', 'l', 'l', 'e', 'c', 't', 'i', 'o', 'n', '\0',
  /*  3801 - "urlbar.prefSuggestNonsponsored" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'N', 'o', 'n', 's', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', '\0',
  /*  3832 - "urlbar.prefSuggestSponsored" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'S', 'p', 'o', 'n', 's', 'o', 'r', 'e', 'd', '\0',
  /*  3860 - "urlbar.prefSuggestTopsites" */ 'u', 'r', 'l', 'b', 'a', 'r', '.', 'p', 'r', 'e', 'f', 'S', 'u', 'g', 'g', 'e', 's', 't', 'T', 'o', 'p', 's', 'i', 't', 'e', 's', '\0',
  /*  3887 - "browserEngagement.activeTicks" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'a', 'c', 't', 'i', 'v', 'e', 'T', 'i', 'c', 'k', 's', '\0',
  /*  3917 - "browserEngagement.profileCount" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'p', 'r', 'o', 'f', 'i', 'l', 'e', 'C', 'o', 'u', 'n', 't', '\0',
  /*  3948 - "browserEngagement.uriCount" */ 'b', 'r', 'o', 'w', 's', 'e', 'r', 'E', 'n', 'g', 'a', 'g', 'e', 'm', 'e', 'n', 't', '.', 'u', 'r', 'i', 'C', 'o', 'u', 'n', 't', '\0',
  /*  3975 - "pingCentre.sendFailures" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  3999 - "pingCentre.sendFailuresByNamespace" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', 'B', 'y', 'N', 'a', 'm', 'e', 's', 'p', 'a', 'c', 'e', '\0',
  /*  4034 - "pingCentre.sendSuccessesByNamespace" */ 'p', 'i', 'n', 'g', 'C', 'e', 'n', 't', 'r', 'e', '.', 's', 'e', 'n', 'd', 'S', 'u', 'c', 'c', 'e', 's', 's', 'e', 's', 'B', 'y', 'N', 'a', 'm', 'e', 's', 'p', 'a', 'c', 'e', '\0',
  /*  4070 - "gmp.updateXmlFetchResult" */ 'g', 'm', 'p', '.', 'u', 'p', 'd', 'a', 't', 'e', 'X', 'm', 'l', 'F', 'e', 't', 'c', 'h', 'R', 'e', 's', 'u', 'l', 't', '\0',
  /*  4095 - "rtcrtpsender.count" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', '\0',
  /*  4114 - "rtcrtpsender.countSetparametersCompat" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'c', 'o', 'u', 'n', 't', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', 'C', 'o', 'm', 'p', 'a', 't', '\0',
  /*  4152 - "rtcrtpsender.usedSendencodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', '.', 'u', 's', 'e', 'd', 'S', 'e', 'n', 'd', 'e', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*  4183 - "rtcrtpsenderSetparameters.blameLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  4228 - "rtcrtpsenderSetparameters.blameNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  4275 - "rtcrtpsenderSetparameters.blameNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'b', 'l', 'a', 'm', 'e', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4322 - "rtcrtpsenderSetparameters.failLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  4366 - "rtcrtpsenderSetparameters.failNoEncodings" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'E', 'n', 'c', 'o', 'd', 'i', 'n', 'g', 's', '\0',
  /*  4408 - "rtcrtpsenderSetparameters.failNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  4454 - "rtcrtpsenderSetparameters.failNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4500 - "rtcrtpsenderSetparameters.failOther" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'O', 't', 'h', 'e', 'r', '\0',
  /*  4536 - "rtcrtpsenderSetparameters.failRidChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'R', 'i', 'd', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  4577 - "rtcrtpsenderSetparameters.failStaleTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'f', 'a', 'i', 'l', 'S', 't', 'a', 'l', 'e', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4626 - "rtcrtpsenderSetparameters.warnLengthChanged" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'L', 'e', 'n', 'g', 't', 'h', 'C', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  4670 - "rtcrtpsenderSetparameters.warnNoGetparameters" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'G', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '\0',
  /*  4716 - "rtcrtpsenderSetparameters.warnNoTransactionid" */ 'r', 't', 'c', 'r', 't', 'p', 's', 'e', 'n', 'd', 'e', 'r', 'S', 'e', 't', 'p', 'a', 'r', 'a', 'm', 'e', 't', 'e', 'r', 's', '.', 'w', 'a', 'r', 'n', 'N', 'o', 'T', 'r', 'a', 'n', 's', 'a', 'c', 't', 'i', 'o', 'n', 'i', 'd', '\0',
  /*  4762 - "bloburl.resolveStopped" */ 'b', 'l', 'o', 'b', 'u', 'r', 'l', '.', 'r', 'e', 's', 'o', 'l', 'v', 'e', 'S', 't', 'o', 'p', 'p', 'e', 'd', '\0',
  /*  4785 - "perf.pageLoad" */ 'p', 'e', 'r', 'f', '.', 'p', 'a', 'g', 'e', 'L', 'o', 'a', 'd', '\0',
  /*  4799 - "paint.buildDisplaylistTime" */ 'p', 'a', 'i', 'n', 't', '.', 'b', 'u', 'i', 'l', 'd', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'l', 'i', 's', 't', 'T', 'i', 'm', 'e', '\0',
  /*  4826 - "wr.framebuildTime" */ 'w', 'r', '.', 'f', 'r', 'a', 'm', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  4844 - "wr.gpuWaitTime" */ 'w', 'r', '.', 'g', 'p', 'u', 'W', 'a', 'i', 't', 'T', 'i', 'm', 'e', '\0',
  /*  4859 - "wr.rasterizeBlobsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'B', 'l', 'o', 'b', 's', 'T', 'i', 'm', 'e', '\0',
  /*  4881 - "wr.rasterizeGlyphsTime" */ 'w', 'r', '.', 'r', 'a', 's', 't', 'e', 'r', 'i', 'z', 'e', 'G', 'l', 'y', 'p', 'h', 's', 'T', 'i', 'm', 'e', '\0',
  /*  4904 - "wr.rendererTime" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', '\0',
  /*  4920 - "wr.rendererTimeNoSc" */ 'w', 'r', '.', 'r', 'e', 'n', 'd', 'e', 'r', 'e', 'r', 'T', 'i', 'm', 'e', 'N', 'o', 'S', 'c', '\0',
  /*  4940 - "wr.scenebuildTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 'b', 'u', 'i', 'l', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  4958 - "wr.sceneswapTime" */ 'w', 'r', '.', 's', 'c', 'e', 'n', 'e', 's', 'w', 'a', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  4975 - "wr.textureCacheUpdateTime" */ 'w', 'r', '.', 't', 'e', 'x', 't', 'u', 'r', 'e', 'C', 'a', 'c', 'h', 'e', 'U', 'p', 'd', 'a', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  5001 - "wr.timeToFrameBuild" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'F', 'r', 'a', 'm', 'e', 'B', 'u', 'i', 'l', 'd', '\0',
  /*  5021 - "wr.timeToRenderStart" */ 'w', 'r', '.', 't', 'i', 'm', 'e', 'T', 'o', 'R', 'e', 'n', 'd', 'e', 'r', 'S', 't', 'a', 'r', 't', '\0',
  /*  5042 - "dotprint.androidDialogRequested" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'a', 'n', 'd', 'r', 'o', 'i', 'd', 'D', 'i', 'a', 'l', 'o', 'g', 'R', 'e', 'q', 'u', 'e', 's', 't', 'e', 'd', '\0',
  /*  5074 - "dotprint.failure" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'f', 'a', 'i', 'l', 'u', 'r', 'e', '\0',
  /*  5091 - "dotprint.requested" */ 'd', 'o', 't', 'p', 'r', 'i', 'n', 't', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 'e', 'd', '\0',
  /*  5110 - "networking.cookieAccessFixupDiff" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'A', 'c', 'c', 'e', 's', 's', 'F', 'i', 'x', 'u', 'p', 'D', 'i', 'f', 'f', '\0',
  /*  5143 - "networking.cookieCreationFixupDiff" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'C', 'r', 'e', 'a', 't', 'i', 'o', 'n', 'F', 'i', 'x', 'u', 'p', 'D', 'i', 'f', 'f', '\0',
  /*  5178 - "networking.cookieTimestampFixedCount" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'c', 'o', 'o', 'k', 'i', 'e', 'T', 'i', 'm', 'e', 's', 't', 'a', 'm', 'p', 'F', 'i', 'x', 'e', 'd', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5215 - "networking.dnsFailedLookupTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'F', 'a', 'i', 'l', 'e', 'd', 'L', 'o', 'o', 'k', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  5246 - "networking.dnsLookupTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'L', 'o', 'o', 'k', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /*  5271 - "networking.dnsRenewalTime" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'R', 'e', 'n', 'e', 'w', 'a', 'l', 'T', 'i', 'm', 'e', '\0',
  /*  5297 - "networking.dnsRenewalTimeForTtl" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'd', 'n', 's', 'R', 'e', 'n', 'e', 'w', 'a', 'l', 'T', 'i', 'm', 'e', 'F', 'o', 'r', 'T', 't', 'l', '\0',
  /*  5329 - "networking.http1DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '1', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  5364 - "networking.http2DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '2', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  5399 - "networking.http3DownloadThroughput" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 'h', 't', 't', 'p', '3', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', 'T', 'h', 'r', 'o', 'u', 'g', 'h', 'p', 'u', 't', '\0',
  /*  5434 - "networking.speculativeConnectOutcome" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', 'i', 'n', 'g', '.', 's', 'p', 'e', 'c', 'u', 'l', 'a', 't', 'i', 'v', 'e', 'C', 'o', 'n', 'n', 'e', 'c', 't', 'O', 'u', 't', 'c', 'o', 'm', 'e', '\0',
  /*  5471 - "netwerk.earlyHints" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'a', 'r', 'l', 'y', 'H', 'i', 'n', 't', 's', '\0',
  /*  5490 - "netwerk.ehLinkType" */ 'n', 'e', 't', 'w', 'e', 'r', 'k', '.', 'e', 'h', 'L', 'i', 'n', 'k', 'T', 'y', 'p', 'e', '\0',
  /*  5509 - "network.corsAuthorizationHeader" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'c', 'o', 'r', 's', 'A', 'u', 't', 'h', 'o', 'r', 'i', 'z', 'a', 't', 'i', 'o', 'n', 'H', 'e', 'a', 'd', 'e', 'r', '\0',
  /*  5541 - "network.dataSizePbPerType" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'd', 'a', 't', 'a', 'S', 'i', 'z', 'e', 'P', 'b', 'P', 'e', 'r', 'T', 'y', 'p', 'e', '\0',
  /*  5567 - "network.dataSizePerType" */ 'n', 'e', 't', 'w', 'o', 'r', 'k', '.', 'd', 'a', 't', 'a', 'S', 'i', 'z', 'e', 'P', 'e', 'r', 'T', 'y', 'p', 'e', '\0',
  /*  5591 - "dataStorage.entries" */ 'd', 'a', 't', 'a', 'S', 't', 'o', 'r', 'a', 'g', 'e', '.', 'e', 'n', 't', 'r', 'i', 'e', 's', '\0',
  /*  5611 - "dataStorage.migration" */ 'd', 'a', 't', 'a', 'S', 't', 'o', 'r', 'a', 'g', 'e', '.', 'm', 'i', 'g', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  5633 - "backgroundTasksRmdirBase.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  5668 - "backgroundTasksRmdirBase.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  5704 - "backgroundTasksRmdirBase.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5742 - "backgroundTasksRmdirBase.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5778 - "backgroundTasksRmdirBase.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  5813 - "backgroundTasksRmdirBase.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  5855 - "backgroundTasksRmdirBase.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  5899 - "backgroundTasksRmdirBase.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'B', 'a', 's', 'e', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  5933 - "backgroundTasksRmdirHttpCache.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  5973 - "backgroundTasksRmdirHttpCache.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  6014 - "backgroundTasksRmdirHttpCache.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6057 - "backgroundTasksRmdirHttpCache.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6098 - "backgroundTasksRmdirHttpCache.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  6138 - "backgroundTasksRmdirHttpCache.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  6185 - "backgroundTasksRmdirHttpCache.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6234 - "backgroundTasksRmdirHttpCache.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'H', 't', 't', 'p', 'C', 'a', 'c', 'h', 'e', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  6273 - "backgroundTasksRmdirQuota.elapsedMs" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', 'M', 's', '\0',
  /*  6309 - "backgroundTasksRmdirQuota.metricBase" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'm', 'e', 't', 'r', 'i', 'c', 'B', 'a', 's', 'e', '\0',
  /*  6346 - "backgroundTasksRmdirQuota.removalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'r', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6385 - "backgroundTasksRmdirQuota.retryCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'r', 'e', 't', 'r', 'y', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6422 - "backgroundTasksRmdirQuota.succeeded" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'c', 'c', 'e', 'e', 'd', 'e', 'd', '\0',
  /*  6458 - "backgroundTasksRmdirQuota.suffixEverFailed" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'f', 'f', 'i', 'x', 'E', 'v', 'e', 'r', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  6501 - "backgroundTasksRmdirQuota.suffixRemovalCount" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 's', 'u', 'f', 'f', 'i', 'x', 'R', 'e', 'm', 'o', 'v', 'a', 'l', 'C', 'o', 'u', 'n', 't', '\0',
  /*  6546 - "backgroundTasksRmdirQuota.wasFirst" */ 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'T', 'a', 's', 'k', 's', 'R', 'm', 'd', 'i', 'r', 'Q', 'u', 'o', 't', 'a', '.', 'w', 'a', 's', 'F', 'i', 'r', 's', 't', '\0',
  /*  6581 - "cookieBanners.normalWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'n', 'o', 'r', 'm', 'a', 'l', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*  6619 - "cookieBanners.privateWindowServiceMode" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'p', 'r', 'i', 'v', 'a', 't', 'e', 'W', 'i', 'n', 'd', 'o', 'w', 'S', 'e', 'r', 'v', 'i', 'c', 'e', 'M', 'o', 'd', 'e', '\0',
  /*  6658 - "cookieBanners.reload" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'e', 'l', 'o', 'a', 'd', '\0',
  /*  6679 - "cookieBanners.ruleLookupByDomain" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'D', 'o', 'm', 'a', 'i', 'n', '\0',
  /*  6712 - "cookieBanners.ruleLookupByLoad" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 'r', 'u', 'l', 'e', 'L', 'o', 'o', 'k', 'u', 'p', 'B', 'y', 'L', 'o', 'a', 'd', '\0',
  /*  6743 - "cookieBanners.serviceDetectOnly" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', '.', 's', 'e', 'r', 'v', 'i', 'c', 'e', 'D', 'e', 't', 'e', 'c', 't', 'O', 'n', 'l', 'y', '\0',
  /*  6775 - "cookieBannersClick.handleDuration" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'h', 'a', 'n', 'd', 'l', 'e', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  6809 - "cookieBannersClick.result" */ 'c', 'o', 'o', 'k', 'i', 'e', 'B', 'a', 'n', 'n', 'e', 'r', 's', 'C', 'l', 'i', 'c', 'k', '.', 'r', 'e', 's', 'u', 'l', 't', '\0',
  /*  6835 - "crash.processType" */ 'c', 'r', 'a', 's', 'h', '.', 'p', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', '\0',
  /*  6853 - "crash.startup" */ 'c', 'r', 'a', 's', 'h', '.', 's', 't', 'a', 'r', 't', 'u', 'p', '\0',
  /*  6867 - "crash.time" */ 'c', 'r', 'a', 's', 'h', '.', 't', 'i', 'm', 'e', '\0',
  /*  6878 - "crash.uptime" */ 'c', 'r', 'a', 's', 'h', '.', 'u', 'p', 't', 'i', 'm', 'e', '\0',
  /*  6891 - "extensions.processEvent" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'p', 'r', 'o', 'c', 'e', 's', 's', 'E', 'v', 'e', 'n', 't', '\0',
  /*  6915 - "extensions.startupCacheLoadTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'L', 'o', 'a', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  6947 - "extensions.useRemotePolicy" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'u', 's', 'e', 'R', 'e', 'm', 'o', 't', 'e', 'P', 'o', 'l', 'i', 'c', 'y', '\0',
  /*  6974 - "extensions.useRemotePref" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', '.', 'u', 's', 'e', 'R', 'e', 'm', 'o', 't', 'e', 'P', 'r', 'e', 'f', '\0',
  /*  6999 - "extensionsApisDnr.evaluateRulesCountMax" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'e', 'v', 'a', 'l', 'u', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'C', 'o', 'u', 'n', 't', 'M', 'a', 'x', '\0',
  /*  7039 - "extensionsApisDnr.evaluateRulesTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'e', 'v', 'a', 'l', 'u', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'T', 'i', 'm', 'e', '\0',
  /*  7075 - "extensionsApisDnr.startupCacheEntries" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'E', 'n', 't', 'r', 'i', 'e', 's', '\0',
  /*  7113 - "extensionsApisDnr.startupCacheReadSize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'R', 'e', 'a', 'd', 'S', 'i', 'z', 'e', '\0',
  /*  7152 - "extensionsApisDnr.startupCacheReadTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'R', 'e', 'a', 'd', 'T', 'i', 'm', 'e', '\0',
  /*  7191 - "extensionsApisDnr.startupCacheWriteSize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'W', 'r', 'i', 't', 'e', 'S', 'i', 'z', 'e', '\0',
  /*  7231 - "extensionsApisDnr.startupCacheWriteTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'C', 'a', 'c', 'h', 'e', 'W', 'r', 'i', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  7271 - "extensionsApisDnr.validateRulesTime" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'A', 'p', 'i', 's', 'D', 'n', 'r', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'e', 'R', 'u', 'l', 'e', 's', 'T', 'i', 'm', 'e', '\0',
  /*  7307 - "extensionsQuarantinedDomains.listhash" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'l', 'i', 's', 't', 'h', 'a', 's', 'h', '\0',
  /*  7345 - "extensionsQuarantinedDomains.listsize" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'l', 'i', 's', 't', 's', 'i', 'z', 'e', '\0',
  /*  7383 - "extensionsQuarantinedDomains.remotehash" */ 'e', 'x', 't', 'e', 'n', 's', 'i', 'o', 'n', 's', 'Q', 'u', 'a', 'r', 'a', 'n', 't', 'i', 'n', 'e', 'd', 'D', 'o', 'm', 'a', 'i', 'n', 's', '.', 'r', 'e', 'm', 'o', 't', 'e', 'h', 'a', 's', 'h', '\0',
  /*  7423 - "formautofillCreditcards.autofillProfilesCount" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'P', 'r', 'o', 'f', 'i', 'l', 'e', 's', 'C', 'o', 'u', 'n', 't', '\0',
  /*  7469 - "formautofillCreditcards.formCleared" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'C', 'l', 'e', 'a', 'r', 'e', 'd', '\0',
  /*  7505 - "formautofillCreditcards.formDetected" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'D', 'e', 't', 'e', 'c', 't', 'e', 'd', '\0',
  /*  7542 - "formautofillCreditcards.formFilled" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'F', 'i', 'l', 'l', 'e', 'd', '\0',
  /*  7577 - "formautofillCreditcards.formFilledModified" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'F', 'i', 'l', 'l', 'e', 'd', 'M', 'o', 'd', 'i', 'f', 'i', 'e', 'd', '\0',
  /*  7620 - "formautofillCreditcards.formPopupShown" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'P', 'o', 'p', 'u', 'p', 'S', 'h', 'o', 'w', 'n', '\0',
  /*  7659 - "formautofillCreditcards.formSubmitted" */ 'f', 'o', 'r', 'm', 'a', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'C', 'r', 'e', 'd', 'i', 't', 'c', 'a', 'r', 'd', 's', '.', 'f', 'o', 'r', 'm', 'S', 'u', 'b', 'm', 'i', 't', 't', 'e', 'd', '\0',
  /*  7697 - "fog.failedIdleRegistration" */ 'f', 'o', 'g', '.', 'f', 'a', 'i', 'l', 'e', 'd', 'I', 'd', 'l', 'e', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', '\0',
  /*  7724 - "fog.initialization" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 'i', 'a', 'l', 'i', 'z', 'a', 't', 'i', 'o', 'n', '\0',
  /*  7743 - "fog.initsDuringShutdown" */ 'f', 'o', 'g', '.', 'i', 'n', 'i', 't', 's', 'D', 'u', 'r', 'i', 'n', 'g', 'S', 'h', 'u', 't', 'd', 'o', 'w', 'n', '\0',
  /*  7767 - "fog.maxPingsPerMinute" */ 'f', 'o', 'g', '.', 'm', 'a', 'x', 'P', 'i', 'n', 'g', 's', 'P', 'e', 'r', 'M', 'i', 'n', 'u', 't', 'e', '\0',
  /*  7789 - "fogIpc.bufferSizes" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'b', 'u', 'f', 'f', 'e', 'r', 'S', 'i', 'z', 'e', 's', '\0',
  /*  7808 - "fogIpc.flushDurations" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'D', 'u', 'r', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /*  7830 - "fogIpc.flushFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'f', 'l', 'u', 's', 'h', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7851 - "fogIpc.replayFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 'r', 'e', 'p', 'l', 'a', 'y', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7873 - "fogIpc.shutdownRegistrationFailures" */ 'f', 'o', 'g', 'I', 'p', 'c', '.', 's', 'h', 'u', 't', 'd', 'o', 'w', 'n', 'R', 'e', 'g', 'i', 's', 't', 'r', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'u', 'r', 'e', 's', '\0',
  /*  7909 - "fogValidation.validateEarlyEvent" */ 'f', 'o', 'g', 'V', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'e', 'E', 'a', 'r', 'l', 'y', 'E', 'v', 'e', 'n', 't', '\0',
  /*  7942 - "testOnly.badCode" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'b', 'a', 'd', 'C', 'o', 'd', 'e', '\0',
  /*  7959 - "testOnly.canWeFlagIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'F', 'l', 'a', 'g', 'I', 't', '\0',
  /*  7980 - "testOnly.canWeTimeIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'a', 'n', 'W', 'e', 'T', 'i', 'm', 'e', 'I', 't', '\0',
  /*  8001 - "testOnly.cheesyString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  8023 - "testOnly.cheesyStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'c', 'h', 'e', 'e', 's', 'y', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  8049 - "testOnly.doYouRemember" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'd', 'o', 'Y', 'o', 'u', 'R', 'e', 'm', 'e', 'm', 'b', 'e', 'r', '\0',
  /*  8072 - "testOnly.mabelsBalloonLabels" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 'L', 'a', 'b', 'e', 'l', 's', '\0',
  /*  8101 - "testOnly.mabelsBalloonStrings" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 'S', 't', 'r', 'i', 'n', 'g', 's', '\0',
  /*  8131 - "testOnly.mabelsBathroomCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'B', 'a', 't', 'h', 'r', 'o', 'o', 'm', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  8163 - "testOnly.mabelsKitchenCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'K', 'i', 't', 'c', 'h', 'e', 'n', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  8194 - "testOnly.mabelsLabelMaker" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'a', 'b', 'e', 'l', 'M', 'a', 'k', 'e', 'r', '\0',
  /*  8220 - "testOnly.mabelsLabeledCounters" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', 's', '\0',
  /*  8251 - "testOnly.mabelsLikeBalloons" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'i', 'k', 'e', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 's', '\0',
  /*  8279 - "testOnly.mabelsLikeLabeledBalloons" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'a', 'b', 'e', 'l', 's', 'L', 'i', 'k', 'e', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'B', 'a', 'l', 'l', 'o', 'o', 'n', 's', '\0',
  /*  8314 - "testOnly.meaningOfLife" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'e', 'a', 'n', 'i', 'n', 'g', 'O', 'f', 'L', 'i', 'f', 'e', '\0',
  /*  8337 - "testOnly.mirrorTime" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', '\0',
  /*  8357 - "testOnly.mirrorTimeNanos" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 'T', 'i', 'm', 'e', 'N', 'a', 'n', 'o', 's', '\0',
  /*  8382 - "testOnly.mirrorsForLabeledBools" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'm', 'i', 'r', 'r', 'o', 'r', 's', 'F', 'o', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'B', 'o', 'o', 'l', 's', '\0',
  /*  8414 - "testOnly.onePingOneBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'o', 'n', 'e', 'P', 'i', 'n', 'g', 'O', 'n', 'e', 'B', 'o', 'o', 'l', '\0',
  /*  8438 - "testOnly.whatADate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'A', 'D', 'a', 't', 'e', '\0',
  /*  8457 - "testOnly.whatIdIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'I', 'd', 'I', 't', '\0',
  /*  8475 - "testOnly.whatTimeIsIt" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', '.', 'w', 'h', 'a', 't', 'T', 'i', 'm', 'e', 'I', 's', 'I', 't', '\0',
  /*  8497 - "testOnlyIpc.aBool" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'B', 'o', 'o', 'l', '\0',
  /*  8515 - "testOnlyIpc.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8536 - "testOnlyIpc.aCustomDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'C', 'u', 's', 't', 'o', 'm', 'D', 'i', 's', 't', '\0',
  /*  8560 - "testOnlyIpc.aDate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'D', 'a', 't', 'e', '\0',
  /*  8578 - "testOnlyIpc.aLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8606 - "testOnlyIpc.aMemoryDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'M', 'e', 'm', 'o', 'r', 'y', 'D', 'i', 's', 't', '\0',
  /*  8630 - "testOnlyIpc.aQuantity" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'Q', 'u', 'a', 'n', 't', 'i', 't', 'y', '\0',
  /*  8652 - "testOnlyIpc.aString" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', '\0',
  /*  8672 - "testOnlyIpc.aStringList" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'S', 't', 'r', 'i', 'n', 'g', 'L', 'i', 's', 't', '\0',
  /*  8696 - "testOnlyIpc.aText" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'T', 'e', 'x', 't', '\0',
  /*  8714 - "testOnlyIpc.aTimingDist" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'T', 'i', 'm', 'i', 'n', 'g', 'D', 'i', 's', 't', '\0',
  /*  8738 - "testOnlyIpc.aUrl" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'r', 'l', '\0',
  /*  8755 - "testOnlyIpc.aUuid" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'U', 'u', 'i', 'd', '\0',
  /*  8773 - "testOnlyIpc.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
  /*  8793 - "testOnlyIpc.anExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  8827 - "testOnlyIpc.anotherLabeledCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'a', 'n', 'o', 't', 'h', 'e', 'r', 'L', 'a', 'b', 'e', 'l', 'e', 'd', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8861 - "testOnlyIpc.eventWithExtra" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'e', 'v', 'e', 'n', 't', 'W', 'i', 't', 'h', 'E', 'x', 't', 'r', 'a', '\0',
  /*  8888 - "testOnlyIpc.irate" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'i', 'r', 'a', 't', 'e', '\0',
  /*  8906 - "testOnlyIpc.noExtraEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'n', 'o', 'E', 'x', 't', 'r', 'a', 'E', 'v', 'e', 'n', 't', '\0',
  /*  8931 - "testOnlyIpc.rateWithExternalDenominator" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'I', 'p', 'c', '.', 'r', 'a', 't', 'e', 'W', 'i', 't', 'h', 'E', 'x', 't', 'e', 'r', 'n', 'a', 'l', 'D', 'e', 'n', 'o', 'm', 'i', 'n', 'a', 't', 'o', 'r', '\0',
  /*  8971 - "testOnlyJog.aCounter" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'C', 'o', 'u', 'n', 't', 'e', 'r', '\0',
  /*  8992 - "testOnlyJog.anEvent" */ 't', 'e', 's', 't', 'O', 'n', 'l', 'y', 'J', 'o', 'g', '.', 'a', 'n', 'E', 'v', 'e', 'n', 't', '\0',
  /*  9012 - "nimbusEvents.enrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  9038 - "nimbusEvents.enrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  9062 - "nimbusEvents.exposure" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'e', 'x', 'p', 'o', 's', 'u', 'r', 'e', '\0',
  /*  9084 - "nimbusEvents.unenrollFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  9112 - "nimbusEvents.unenrollment" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'u', 'n', 'e', 'n', 'r', 'o', 'l', 'l', 'm', 'e', 'n', 't', '\0',
  /*  9138 - "nimbusEvents.validationFailed" */ 'n', 'i', 'm', 'b', 'u', 's', 'E', 'v', 'e', 'n', 't', 's', '.', 'v', 'a', 'l', 'i', 'd', 'a', 't', 'i', 'o', 'n', 'F', 'a', 'i', 'l', 'e', 'd', '\0',
  /*  9168 - "pwmgr.formAutofillResult" */ 'p', 'w', 'm', 'g', 'r', '.', 'f', 'o', 'r', 'm', 'A', 'u', 't', 'o', 'f', 'i', 'l', 'l', 'R', 'e', 's', 'u', 'l', 't', '\0',
  /*  9193 - "pdfjs.buttons" */ 'p', 'd', 'f', 'j', 's', '.', 'b', 'u', 't', 't', 'o', 'n', 's', '\0',
  /*  9207 - "pdfjs.editing" */ 'p', 'd', 'f', 'j', 's', '.', 'e', 'd', 'i', 't', 'i', 'n', 'g', '\0',
  /*  9221 - "pdfjs.geckoview" */ 'p', 'd', 'f', 'j', 's', '.', 'g', 'e', 'c', 'k', 'o', 'v', 'i', 'e', 'w', '\0',
  /*  9237 - "pdfjs.stamp" */ 'p', 'd', 'f', 'j', 's', '.', 's', 't', 'a', 'm', 'p', '\0',
  /*  9249 - "pdfjs.timeToView" */ 'p', 'd', 'f', 'j', 's', '.', 't', 'i', 'm', 'e', 'T', 'o', 'V', 'i', 'e', 'w', '\0',
  /*  9266 - "pdfjs.used" */ 'p', 'd', 'f', 'j', 's', '.', 'u', 's', 'e', 'd', '\0',
  /*  9277 - "power.cpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  9302 - "power.cpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  9332 - "power.cpuTimePerTrackerTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'c', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'T', 'r', 'a', 'c', 'k', 'e', 'r', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  9362 - "power.gpuTimeBogusValues" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'B', 'o', 'g', 'u', 's', 'V', 'a', 'l', 'u', 'e', 's', '\0',
  /*  9387 - "power.gpuTimePerProcessTypeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 'g', 'p', 'u', 'T', 'i', 'm', 'e', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', 'M', 's', '\0',
  /*  9417 - "power.totalCpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'C', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  9438 - "power.totalGpuTimeMs" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'G', 'p', 'u', 'T', 'i', 'm', 'e', 'M', 's', '\0',
  /*  9459 - "power.totalThreadWakeups" */ 'p', 'o', 'w', 'e', 'r', '.', 't', 'o', 't', 'a', 'l', 'T', 'h', 'r', 'e', 'a', 'd', 'W', 'a', 'k', 'e', 'u', 'p', 's', '\0',
  /*  9484 - "power.wakeupsPerProcessType" */ 'p', 'o', 'w', 'e', 'r', '.', 'w', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'P', 'r', 'o', 'c', 'e', 's', 's', 'T', 'y', 'p', 'e', '\0',
  /*  9512 - "powerBattery.percentageWhenUserActive" */ 'p', 'o', 'w', 'e', 'r', 'B', 'a', 't', 't', 'e', 'r', 'y', '.', 'p', 'e', 'r', 'c', 'e', 'n', 't', 'a', 'g', 'e', 'W', 'h', 'e', 'n', 'U', 's', 'e', 'r', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9550 - "powerCpuMsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9588 - "powerCpuMsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9626 - "powerCpuMsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  9657 - "powerCpuMsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9690 - "powerCpuMsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'C', 'p', 'u', 'M', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9725 - "powerWakeupsPerThread.contentBackground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'B', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9765 - "powerWakeupsPerThread.contentForeground" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'c', 'o', 'n', 't', 'e', 'n', 't', 'F', 'o', 'r', 'e', 'g', 'r', 'o', 'u', 'n', 'd', '\0',
  /*  9805 - "powerWakeupsPerThread.gpuProcess" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'g', 'p', 'u', 'P', 'r', 'o', 'c', 'e', 's', 's', '\0',
  /*  9838 - "powerWakeupsPerThread.parentActive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'A', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9873 - "powerWakeupsPerThread.parentInactive" */ 'p', 'o', 'w', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', 's', 'P', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 'p', 'a', 'r', 'e', 'n', 't', 'I', 'n', 'a', 'c', 't', 'i', 'v', 'e', '\0',
  /*  9910 - "fingerprintingProtection.canvasNoiseCalculateTime" */ 'f', 'i', 'n', 'g', 'e', 'r', 'p', 'r', 'i', 'n', 't', 'i', 'n', 'g', 'P', 'r', 'o', 't', 'e', 'c', 't', 'i', 'o', 'n', '.', 'c', 'a', 'n', 'v', 'a', 's', 'N', 'o', 'i', 's', 'e', 'C', 'a', 'l', 'c', 'u', 'l', 'a', 't', 'e', 'T', 'i', 'm', 'e', '\0',
  /*  9960 - "searchEngineDefault.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /*  9988 - "searchEngineDefault.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /* 10020 - "searchEngineDefault.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /* 10049 - "searchEngineDefault.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /* 10078 - "searchEngineDefault.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /* 10112 - "searchEngineDefault.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'D', 'e', 'f', 'a', 'u', 'l', 't', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /* 10141 - "searchEnginePrivate.changed" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'd', '\0',
  /* 10169 - "searchEnginePrivate.displayName" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'd', 'i', 's', 'p', 'l', 'a', 'y', 'N', 'a', 'm', 'e', '\0',
  /* 10201 - "searchEnginePrivate.engineId" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'e', 'n', 'g', 'i', 'n', 'e', 'I', 'd', '\0',
  /* 10230 - "searchEnginePrivate.loadPath" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'l', 'o', 'a', 'd', 'P', 'a', 't', 'h', '\0',
  /* 10259 - "searchEnginePrivate.submissionUrl" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 's', 'u', 'b', 'm', 'i', 's', 's', 'i', 'o', 'n', 'U', 'r', 'l', '\0',
  /* 10293 - "searchEnginePrivate.verified" */ 's', 'e', 'a', 'r', 'c', 'h', 'E', 'n', 'g', 'i', 'n', 'e', 'P', 'r', 'i', 'v', 'a', 't', 'e', '.', 'v', 'e', 'r', 'i', 'f', 'i', 'e', 'd', '\0',
  /* 10322 - "searchService.startupTime" */ 's', 'e', 'a', 'r', 'c', 'h', 'S', 'e', 'r', 'v', 'i', 'c', 'e', '.', 's', 't', 'a', 'r', 't', 'u', 'p', 'T', 'i', 'm', 'e', '\0',
  /* 10348 - "dap.reportGenerationStatus" */ 'd', 'a', 'p', '.', 'r', 'e', 'p', 'o', 'r', 't', 'G', 'e', 'n', 'e', 'r', 'a', 't', 'i', 'o', 'n', 'S', 't', 'a', 't', 'u', 's', '\0',
  /* 10375 - "dap.uploadStatus" */ 'd', 'a', 'p', '.', 'u', 'p', 'l', 'o', 'a', 'd', 'S', 't', 'a', 't', 'u', 's', '\0',
  /* 10392 - "legacyTelemetry.clientId" */ 'l', 'e', 'g', 'a', 'c', 'y', 'T', 'e', 'l', 'e', 'm', 'e', 't', 'r', 'y', '.', 'c', 'l', 'i', 'e', 'n', 't', 'I', 'd', '\0',
  /* 10417 - "translations.error" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'e', 'r', 'r', 'o', 'r', '\0',
  /* 10436 - "translations.errorRate" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'e', 'r', 'r', 'o', 'r', 'R', 'a', 't', 'e', '\0',
  /* 10459 - "translations.requestsCount" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'r', 'e', 'q', 'u', 'e', 's', 't', 's', 'C', 'o', 'u', 'n', 't', '\0',
  /* 10486 - "translations.restorePage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 'r', 'e', 's', 't', 'o', 'r', 'e', 'P', 'a', 'g', 'e', '\0',
  /* 10511 - "translations.translationRequest" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '.', 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 'R', 'e', 'q', 'u', 'e', 's', 't', '\0',
  /* 10543 - "translationsPanel.aboutTranslations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'b', 'o', 'u', 't', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /* 10579 - "translationsPanel.alwaysOfferTranslations" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'l', 'w', 'a', 'y', 's', 'O', 'f', 'f', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', '\0',
  /* 10621 - "translationsPanel.alwaysTranslateLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'a', 'l', 'w', 'a', 'y', 's', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10663 - "translationsPanel.cancelButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'a', 'n', 'c', 'e', 'l', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10694 - "translationsPanel.changeFromLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10731 - "translationsPanel.changeSourceLanguageButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'S', 'o', 'u', 'r', 'c', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10776 - "translationsPanel.changeToLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'h', 'a', 'n', 'g', 'e', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 10811 - "translationsPanel.close" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', '\0',
  /* 10835 - "translationsPanel.closeFromLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10875 - "translationsPanel.closeSettingsMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'M', 'e', 'n', 'u', '\0',
  /* 10911 - "translationsPanel.closeToLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'c', 'l', 'o', 's', 'e', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 10949 - "translationsPanel.dismissErrorButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'd', 'i', 's', 'm', 'i', 's', 's', 'E', 'r', 'r', 'o', 'r', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 10986 - "translationsPanel.learnMore" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'l', 'e', 'a', 'r', 'n', 'M', 'o', 'r', 'e', '\0',
  /* 11014 - "translationsPanel.manageLanguages" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'm', 'a', 'n', 'a', 'g', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 's', '\0',
  /* 11048 - "translationsPanel.neverTranslateLanguage" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'n', 'e', 'v', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', '\0',
  /* 11089 - "translationsPanel.neverTranslateSite" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'n', 'e', 'v', 'e', 'r', 'T', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'S', 'i', 't', 'e', '\0',
  /* 11126 - "translationsPanel.open" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', '\0',
  /* 11149 - "translationsPanel.openFromLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'F', 'r', 'o', 'm', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 11188 - "translationsPanel.openSettingsMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'S', 'e', 't', 't', 'i', 'n', 'g', 's', 'M', 'e', 'n', 'u', '\0',
  /* 11223 - "translationsPanel.openToLanguageMenu" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'o', 'p', 'e', 'n', 'T', 'o', 'L', 'a', 'n', 'g', 'u', 'a', 'g', 'e', 'M', 'e', 'n', 'u', '\0',
  /* 11260 - "translationsPanel.restorePageButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 'r', 'e', 's', 't', 'o', 'r', 'e', 'P', 'a', 'g', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 11296 - "translationsPanel.translateButton" */ 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'i', 'o', 'n', 's', 'P', 'a', 'n', 'e', 'l', '.', 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'e', 'B', 'u', 't', 't', 'o', 'n', '\0',
  /* 11330 - "firstStartup.deleteTasksTime" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'd', 'e', 'l', 'e', 't', 'e', 'T', 'a', 's', 'k', 's', 'T', 'i', 'm', 'e', '\0',
  /* 11359 - "firstStartup.elapsed" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'e', 'l', 'a', 'p', 's', 'e', 'd', '\0',
  /* 11380 - "firstStartup.normandyInitTime" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 'n', 'o', 'r', 'm', 'a', 'n', 'd', 'y', 'I', 'n', 'i', 't', 'T', 'i', 'm', 'e', '\0',
  /* 11410 - "firstStartup.statusCode" */ 'f', 'i', 'r', 's', 't', 'S', 't', 'a', 'r', 't', 'u', 'p', '.', 's', 't', 'a', 't', 'u', 's', 'C', 'o', 'd', 'e', '\0',
  /* 11434 - "blocklist.addonBlockChange" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'a', 'd', 'd', 'o', 'n', 'B', 'l', 'o', 'c', 'k', 'C', 'h', 'a', 'n', 'g', 'e', '\0',
  /* 11461 - "blocklist.lastModifiedRsAddonsMblf" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'l', 'a', 's', 't', 'M', 'o', 'd', 'i', 'f', 'i', 'e', 'd', 'R', 's', 'A', 'd', 'd', 'o', 'n', 's', 'M', 'b', 'l', 'f', '\0',
  /* 11496 - "blocklist.mlbfGenerationTime" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'G', 'e', 'n', 'e', 'r', 'a', 't', 'i', 'o', 'n', 'T', 'i', 'm', 'e', '\0',
  /* 11525 - "blocklist.mlbfSource" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 'o', 'u', 'r', 'c', 'e', '\0',
  /* 11546 - "blocklist.mlbfStashTimeNewest" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 't', 'a', 's', 'h', 'T', 'i', 'm', 'e', 'N', 'e', 'w', 'e', 's', 't', '\0',
  /* 11576 - "blocklist.mlbfStashTimeOldest" */ 'b', 'l', 'o', 'c', 'k', 'l', 'i', 's', 't', '.', 'm', 'l', 'b', 'f', 'S', 't', 'a', 's', 'h', 'T', 'i', 'm', 'e', 'O', 'l', 'd', 'e', 's', 't', '\0',
  /* 11606 - "update.autoDownload" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'a', 'u', 't', 'o', 'D', 'o', 'w', 'n', 'l', 'o', 'a', 'd', '\0',
  /* 11626 - "update.backgroundUpdate" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'b', 'a', 'c', 'k', 'g', 'r', 'o', 'u', 'n', 'd', 'U', 'p', 'd', 'a', 't', 'e', '\0',
  /* 11650 - "update.canUsuallyApplyUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'A', 'p', 'p', 'l', 'y', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11680 - "update.canUsuallyCheckForUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'C', 'h', 'e', 'c', 'k', 'F', 'o', 'r', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11713 - "update.canUsuallyStageUpdates" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'S', 't', 'a', 'g', 'e', 'U', 'p', 'd', 'a', 't', 'e', 's', '\0',
  /* 11743 - "update.canUsuallyUseBits" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'a', 'n', 'U', 's', 'u', 'a', 'l', 'l', 'y', 'U', 's', 'e', 'B', 'i', 't', 's', '\0',
  /* 11768 - "update.channel" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'c', 'h', 'a', 'n', 'n', 'e', 'l', '\0',
  /* 11783 - "update.enabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 'e', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /* 11798 - "update.serviceEnabled" */ 'u', 'p', 'd', 'a', 't', 'e', '.', 's', 'e', 'r', 'v', 'i', 'c', 'e', 'E', 'n', 'a', 'b', 'l', 'e', 'd', '\0',
  /* 11820 - "timerThread.timersFiredPerWakeup" */ 't', 'i', 'm', 'e', 'r', 'T', 'h', 'r', 'e', 'a', 'd', '.', 't', 'i', 'm', 'e', 'r', 's', 'F', 'i', 'r', 'e', 'd', 'P', 'e', 'r', 'W', 'a', 'k', 'e', 'u', 'p', '\0',
};


static_assert(sizeof(gMetricStringTable) < 4294967296, "Metric string table is too large.");

const metric_entry_t sMetricByNameLookupEntries[] = {
  5764608944668419406ull,
  9223372981747587509ull,
  9223373222265757755ull,
  4035226816607168758ull,
  3458766167382961218ull,
  2882303967675549154ull,
  6917529736310690540ull,
  4035226863851809403ull,
  5764608875948942313ull,
  4035225717095533422ull,
  2305843949811538322ull,
  3458764565360148855ull,
  11529216347443569336ull,
  4035226597563835238ull,
  4035226893916580726ull,
  4035225961908671153ull,
  6917529749195592488ull,
  4035226352750697963ull,
  2882304006330255076ull,
  2882304298388033001ull,
  3458766171677928543ull,
  2882304255438359794ull,
  5764608910308680820ull,
  5764608167279333555ull,
  2882304740769667763ull,
  2305843597624217364ull,
  2882303954790647159ull,
  4035226601858802556ull,
  1152923183939071208ull,
  5764608871653974992ull,
  4035226889621613393ull,
  4035226829492070739ull,
  1729382557557983191ull,
  2882304079344699428ull,
  2305843073638203887ull,
  4611686134391505756ull,
  2305843279796635475ull,
  4035225708505598756ull,
  4035225682735794762ull,
  2305844735790558727ull,
  4035225687030762091ull,
  3458765424353613953ull,
  2305844018531015486ull,
  5764608545236458403ull,
  4035226859556842067ull,
  6917530067023174719ull,
  1729382364284453677ull,
  2882305269050648566ull,
  2305843266911733533ull,
  4035226760772593565ull,
  4035225729980435434ull,
  4035225558181742513ull,
  3458765355634136577ull,
  3458766175972895860ull,
  576461005706495752ull,
  3458765574677470463ull,
  2882303851711431356ull,
  4035225652671023456ull,
  576461903354666839ull,
  5764608330488091962ull,
  5764608162984366212ull,
  1729382759421447455ull,
  9223372986042554843ull,
  3458764853122959480ull,
  9799833991749050533ull,
  2882304281208163724ull,
  2305844014236048163ull,
  5764608768574759298ull,
  5764608970438223397ull,
  2305843915451799642ull,
  8070451185082962190ull,
  4035225532411938630ull,
  2305844727200624095ull,
  2882303950495679827ull,
  576460756598390784ull,
  2305843043573432544ull,
  1152921551851487566ull,
  4035226588973900596ull,
  2882305445144308997ull,
  5188147170162772427ull,
  5764608296128353338ull,
  2882303886071169952ull,
  6341068807913606607ull,
  2305843906861864954ull,
  1152923179644103877ull,
  6917529740605657851ull,
  4035226833787038077ull,
  5188148162300224658ull,
  6917529813620102295ull,
  5764608983323125393ull,
  2305843941221603642ull,
  5764608141509529574ull,
  2305843902566897618ull,
  9799833987454083217ull,
  8646912049055536118ull,
  3458765450123417957ull,
  7493990407009734674ull,
  4035226339865795958ull,
  5764607699127895280ull,
  4035226331275861293ull,
  6917529401303238948ull,
  3458765368519038574ull,
  3458765402878777257ull,
  9799833789885586179ull,
  4035225734275402774ull,
  2882304302683000320ull,
  7493991326132742363ull,
  7493990402714767359ull,
  5764608334783059295ull,
  5764608979028158062ull,
  3458765437238515953ull,
  8646912701890569512ull,
  4611686173046211697ull,
  7493991068434702937ull,
  6917529774965396381ull,
  4035226911096450044ull,
  3458765085051195042ull,
  4035226563204096669ull,
  6917530298951410186ull,
  4035225541001873250ull,
  6917529809325134974ull,
  2305844220394479838ull,
  2305843868207158930ull,
  2882303997740320446ull,
  2882304268323261750ull,
  2882303941905745164ull,
  3458765432943548618ull,
  2305843992761211589ull,
  4035225635491154194ull,
  9223372874373404139ull,
  9799833781295651550ull,
  4035225742865337446ull,
  4035225828764683904ull,
  4035226876736711402ull,
  4611687285442748920ull,
  4035225712800566093ull,
  4035225824469716590ull,
  5764608957553321401ull,
  4035225601131415652ull,
  5764608936078484748ull,
  1729382398644192283ull,
  5764608948963386740ull,
  8646913019718151724ull,
  1152922492449331923ull,
  8070451215147733566ull,
  4035225631196186881ull,
  5188147896012250817ull,
  9799833884374867500ull,
  5764607669063124024ull,
  2882303963380581824ull,
  2882305234690909956ull,
  8070451202262831544ull,
  4035226820902136079ull,
  2882305264755681241ull,
  5764608339078026610ull,
  6341069787166156819ull,
  2882303920430908555ull,
  4035226872441744069ull,
  5188147891717283499ull,
  9223373243740594366ull,
  2305843069343236555ull,
  5764608974733190733ull,
  2305843352811079814ull,
  3458764582540018192ull,
  4035226936866253994ull,
  4035225626901219559ull,
  5188148149415322685ull,
  2882305260460713913ull,
  2305843872502126261ull,
  6917529744900625169ull,
  1729382351399551705ull,
  2882304057869862851ull,
  5188147887422316182ull,
  8070451163608125496ull,
  5188148170890159305ull,
  4035225678440827440ull,
  5188147376321204103ull,
  576460782368194759ull,
  1152922745852404080ull,
  3458766180267863186ull,
  5764608158689398871ull,
  6917530071318142055ull,
  5764608961848288730ull,
  2882304083639666740ull,
  5188146908169765874ull,
  6917529732015723226ull,
  8070451180787994850ull,
  4035225614016317600ull,
  2305843490250034313ull,
  5764608519466654443ull,
  2305843936926636310ull,
  4035225549591807878ull,
  5188147363436302127ull,
  4035225644081088832ull,
  2305843589034282713ull,
  5764608888833844245ull,
  2882303873186267974ull,
  2305843881092060939ull,
  4035226850966907416ull,
  4035226606153769880ull,
  2305843558969511472ull,
  2305844722905656769ull,
  4035225648376056144ull,
  1152923192529005850ull,
  2882303847416464037ull,
  8646912053350503447ull,
  8646912654645928993ull,
  4035226855261874747ull,
  5188146942529504464ull,
  9223373226560725079ull,
  2305843030688530570ull,
  4035225721390500756ull,
  2882303860301366010ull,
  8070451189377929528ull,
  576462015023817184ull,
  8070451210852766226ull,
  5764608695560314851ull,
  3458765707821457530ull,
  2305844100135394833ull,
  5764608691265347523ull,
  1729383790213605528ull,
  8070451197967864212ull,
  3458765398583809918ull,
  3458765364224071240ull,
  2305843593329250040ull,
  2882305243280844609ull,
  576460833907802752ull,
  3458765617627143767ull,
  5764608343372993925ull,
  1152921513196781604ull,
  2305843584739315384ull,
  6917530548059514962ull,
  4035225674145860121ull,
  5764608347667961253ull,
  2882303929020843193ull,
  8646912083415274740ull,
  4611686121506603795ull,
  4035226906801482711ull,
  10376294567958813641ull,
  4035225833059651216ull,
  2882303894661104598ull,
  4035226610448737202ull,
  4035226584678933280ull,
  4035225704210631415ull,
  4035225661260958122ull,
  3458765566087535793ull,
  6917530045548338047ull,
  5188147904602185478ull,
  4035225669850892774ull,
  6341068588870273038ull,
  10376294649563192941ull,
  4035226146592266069ull,
  8070452074141198532ull,
  2882304002035287766ull,
  1152921560441422243ull,
  4035226868146776735ull,
  2882305251870779264ull,
  6917529770670429065ull,
  4035226348455730628ull,
  2305844714315722114ull,
  2305843494545001644ull,
  3458764823058188284ull,
  2305844705725787478ull,
  2882304259733327114ull,
  5764608953258354074ull,
  4035226571794031306ull,
  3458765381403940575ull,
  2305844718610689440ull,
  3458765389993875245ull,
  5188147462220550810ull,
  5764608906013713494ull,
  5764608278948484050ull,
  2305843047868399884ull,
  5188148175185126622ull,
  4035226344160763289ull,
  3458765767950999990ull,
  5188148179480093939ull,
  4035226881031678726ull,
  2882304066459797482ull,
  5764608137214562242ull,
  3458765110820998989ull,
  5764608880243909623ull,
  4035226735002789608ull,
  6917529817915069617ull,
  2882303959085614490ull,
  2882304809489144971ull,
  2882303984855418484ull,
  4035226838082005415ull,
  6917529762080494430ull,
  4035226898211548045ull,
  2882303946200712496ull,
  5764608132919594911ull,
  2882305483799014904ull,
  4035225738570370117ull,
  10376294696807833457ull,
  6341069550942954018ull,
  6917529757785527116ull,
  4035226550319194693ull,
  2305844710020754794ull,
  2882304908273393473ull,
  6917530058433240048ull,
  10376294576548748311ull,
  2305844147380035351ull,
  4035225699915664090ull,
  8646912079120307409ull,
  10376294791297114526ull,
  10952755470586093570ull,
  2882303924725875872ull,
  5764608704150249500ull,
  4035225656965990795ull,
  6917529805030167647ull,
  5764608815819399803ull,
  2882303976265483823ull,
  6917530492224939702ull,
  4035225339138409010ull,
  6917529727720755903ull,
  4035226593268867918ull,
  5764608485106915896ull,
  5188147522350093234ull,
  576460829612835421ull,
  4035226825197103407ull,
  4035226219606710786ull,
  8070451842212963043ull,
  5188148084990812939ull,
  5188148145120355378ull,
  4035226915391417376ull,
  4035226180952004773ull,
  8646912087710242071ull,
  5764608884538876933ull,
  5188147530940027875ull,
  2882303971970516493ull,
  2882303787286921387ull,
  2305843520314805583ull,
  4035226112232527396ull,
  4035225691325729432ull,
  8646912521501942104ull,
  2882305277640583221ull,
  5764608480811948567ull,
  5188148003386433859ull,
  4035226842376972742ull,
  4035226846671940075ull,
  3458765415763679273ull,
  10952755457701191589ull,
  5764607742077568589ull,
  9799833931619508012ull,
  1729383536810533427ull,
  6917530140037619328ull,
  4035226902506515380ull,
  4035225820174749275ull,
  5764609052042602631ull,
  2305843975581342295ull,
  5188147372026236780ull,
  1152923196823973176ull,
  3458765527432829783ull,
  6917529998303697527ull,
  2305843296976504722ull,
  4035225725685468097ull,
  6341069761396352862ull,
  576460795253096758ull,
  4035225515232069370ull,
  5764608356257895895ull,
  2882303911840973914ull,
  5764608351962928575ull,
  2305844740085526038ull,
  4035226885326646056ull,
  4035226395700371173ull,
  8070451219442700908ull,
  5764608918898615467ull,
  5764608966143256061ull,
  8070451193672896870ull,
  5188147870242446911ull,
  2305843288386570100ull,
  4035225618311284923ull,
  5764609047747635308ull,
  4035225498052200112ull,
  10952755453406224264ull,
  4035225695620696758ull,
  4035225592541481028ull,
  2882304818079079639ull,
  5764608497991817881ull,
  2882303890366137274ull,
  4035226335570828625ull,
  576460979936691865ull,
  1152922720082600182ull,
  1729382269795172429ull,
  8070451206557798881ull,
  4035225528116971314ull,
  2305843339926177875ull,
  2882305238985877284ull,
  6917529753490559800ull,
  2882304285503131043ull,
  2882304306977967636ull,
  2305843498839968970ull,
  1729383476680990985ull,
  6917530251706769691ull,
  4035225622606252242ull,
  2882305019942543820ull,
  2305843026393563239ull,
  2305844237574349105ull,
  5188146890989896569ull,
  6917529766375461743ull,
  4035226803722266801ull,
  3458765042101521849ull,
  2305843524609772917ull
};



Maybe<uint32_t>
MetricByNameLookup(const nsACString& aKey)
{
  static const uint16_t BASES[] = {
       2,   0,   5,   0,   2,   1,   0,   0,   2,   2,   0,   3,   4,   1,   0,   0,
       1,   0,   2,   0,   0,   0,   1,   1,   1,   2,   0,   2,   2,   1,   1,   0,
       2,   1,   2,   2,   1,   1,   0,   1,   0,   3,   1,   3,   3,   0,   0,   0,
       0,   0,   0,   0,   3,   0,   0,   1,   0,   0,   0,   0,   0,   0,   7,   2,
       5,   3,   3,   0,   5,   0,   2,   0,   0,   1,   3,   0,   0,   0,   1,   0,
       0,   1,   1,   1,   1,   1,   0,   0,   0,   1,   0,   0,   2,   0,   0,   1,
       2,   3,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   2,   0,   1,   0,
       1,   1,   0,   1,   1,   0,   2,   1,   0,  12,   0,   0,   0,   2,   3,   0,
       0,   2,   0,   3,   1,   2,   0,   0,   1,   4,   0,   0,   2,   0,   1,   4,
       1,   5,   0,   1,  13,   0,   0,   1,   0,   0,   0,   0,   0,   1,   4,   0,
       0,   0,   0,   0,   0,   1,   0,   1,   0,   0,   8,   1,   2,   0,   1,   1,
       0,   1,   0,   2,   1,   0,   0,   0,   1,   0,   3,   1,   0,   4,   1,   3,
      13,   0,   0,   0,   0,   0,   4,   1,   0,   8,   0,   0,   3,   0,   0,   1,
       1,   0,   0,   0,   0,   0,   3,   5,   0,   7,   0,   4,   1,   3,   1,   0,
       0,   0,   1,   1,   2,   1,   0,  10,   0,   0,   0,   1,   1,   0,   1,   9,
       5,   1,   1,   6,   0,   5,   2,   7,   3,   0,   0,   0,   0,   3,   6,  10,
       3,   3,   0,   1,   0,   0,   0,   1,   1,   1,   3,   2,   3,   6,   2,   5,
       2,   0,   0,   5,   1,   0,   0,   8,   2,   2,   0,   3,   0,   1,   0,   3,
       3,   1,   0,   0,   1,   0,   2,   0,   0,   0,   4,   0,   4,   1,   0,   2,
       1,   6,   0,   0,   0,   1,   0,   0,   0,   1,   3,   0,   0,   1,   0,   0,
      13,   0,   1,   4,   2,   0,   2,   0,   1,   4,   0,   1,   0,   0,   0,   0,
       0,   1,   3,   6,   0,   0,   0,   0,   3,   0,   0,   2,   6,   7,   1,   1,
      10,   2,   4,   0,   0,  17,   1,   1,   3,   0,   5,  15,   1,   0,  15,   0,
       1,   1,   6,   0,   6,   0,   9,   4,   4,  11,   0,   1,   7,   0,   0,   0,
       7,  13,   3,   3,   0,  25,   2,   0,  35,   0,  11,   0,   7,   4,  42,   0,
      10,   6,   0,   1,  18,   2,   2,   0,   3,   0,   0,   0,  23,   5,   0,   0,
       0,   0,   1,  24,   3,   0,   7,   2,   0,   2,   0,   6,   6,   0,   1,   6,
       0,  10,   0,   0,   0,   0,   0,  15,   0,   0,   0,   2,   0,  17,   0,   0,
       3,   1,   9,   0,   1,  37,   0,   2,   0,   0,  24,   2,  64,  74,   0,   2,
       3,   0,  26,   0,   2,   0,  16,   0,  19,   0,  14,   0,   0,  12,   2,   0,
       0,   0,  50,   0,   0,   0,  20,   0,  24,  10,   5,   0,   3,   4,   1,   0,
      87,   0,   3,   0, 101,   0,  32, 254,   0,  19,   0,   0,   0,   0,  33, 732,
  };
  

  const char* bytes = aKey.BeginReading();
  size_t length = aKey.Length();
  auto& entry = mozilla::perfecthash::Lookup(bytes, length, BASES,
                                             sMetricByNameLookupEntries);
  return metric_result_check(aKey, entry);
}


/**
 * Get a category's name from the string table.
 */
const char* GetCategoryName(category_entry_t entry) {
  MOZ_ASSERT(entry < sizeof(gCategoryStringTable), "Entry identifier offset larger than string table");
  return &gCategoryStringTable[entry];
}

/**
 * Get a metric's identifier from the string table.
 */
const char* GetMetricIdentifier(metric_entry_t entry) {
  uint32_t offset = GLEAN_OFFSET(entry);
  MOZ_ASSERT(offset < sizeof(gMetricStringTable), "Entry identifier offset larger than string table");
  return &gMetricStringTable[offset];
}

/**
 * Check that the found entry is pointing to the right key
 * and return it.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> category_result_check(const nsACString& aKey, category_entry_t entry) {
  if (MOZ_UNLIKELY(entry > sizeof(gCategoryStringTable))) {
    return Nothing();
  }
  if (aKey.EqualsASCII(gCategoryStringTable + entry)) {
    return Some(entry);
  }
  return Nothing();
}

/**
 * Check if the found entry index is pointing to the right key
 * and return the corresponding metric ID.
 * Or return `Nothing()` if the entry was not found.
 */
static Maybe<uint32_t> metric_result_check(const nsACString& aKey, uint64_t entry) {
  uint32_t metricId = entry >> GLEAN_INDEX_BITS;
  uint32_t offset = GLEAN_OFFSET(entry);

  if (offset > sizeof(gMetricStringTable)) {
    return Nothing();
  }

  if (aKey.EqualsASCII(gMetricStringTable + offset)) {
    return Some(metricId);
  }

  return Nothing();
}


#undef GLEAN_INDEX_BITS
#undef GLEAN_ID_BITS
#undef GLEAN_TYPE_ID
#undef GLEAN_METRIC_ID
#undef GLEAN_OFFSET

} // namespace mozilla::glean
